#!/usr/bin/env bash
#!/bin/env bash

## Name: easy_pipeline.sh
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Ready-to-use pipeline, with no given GFF by default. Variables should be modified properly.


echo "Time - start:"
date

########################################
## VARIABLES
########################################
## main working directory 
PREFIX=~/toto
#PREFIX=/home/user/toto

## RNAprof source folder
SRC=~/RNAprof/src

## mapping description
DESC=desc.cfg

## annotation
GFF=
#GFF=TAIR10.gff

## using gffprof
#GFFPROF=
GFFPROF="--gffprof"

## library-type
#LIBTYPE=fr-unstranded
LIBTYPE=fr-firststrand
#LIBTYPE=fr-secondstrand

## sequencing type
SEQTYPE=
#SEQTYPE="--Pair"

## maximum insert size
MIS=
#MIS=1000

## maximum number of processes
MAXPROC=24

## indexing
INDEX=
#INDEX="--indexing"

########################################
## PIPELINE
########################################
if [ ! -d $SRC ]
then
    echo "Error: $SRC not found"
    exit 1
fi
if [ ! $GFF ]
then
    GFFPROF="--gffprof"
fi

PYTHON=python
WHOLEDE=

$SHELL $SRC/configure.sh --prefix=$PREFIX --mapping=$DESC --library-type=$LIBTYPE --gff=$GFF --MaxInsertSize=$MIS --CPU=$MAXPROC $SEQTYPE $INDEX $WHOLEDE
echo "Time - configure:"
date
if [ $? -eq 0 ]
then
    $SHELL $SRC/preprof.sh --prefix=$PREFIX --CPU=$MAXPROC $SEQTYPE
    echo "Time - preprof:"
    date
    if [ $? -eq 0 ]
    then
	if [ "$GFFPROF" ]
	then
		$SHELL $SRC/GFFprof.sh --prefix=$PREFIX --CPU=$MAXPROC --gff=$GFF
	fi
        echo "Time - GFFprof:"
        date
	if [ $? -eq 0 ]
	then
	    $SHELL $SRC/coverage.sh --prefix=$PREFIX --CPU=$MAXPROC $GFFPROF $WHOLEDE
            echo "Time - coverage:"
            date
	    if [ $? -eq 0 ]
	    then
		$SHELL $SRC/RNAprof.sh --prefix=$PREFIX --CPU=$MAXPROC
                echo "Time - RNAprof:"
                date
		if [ $? -eq 0 ]
                then
                    i=0
                    for res in $PREFIX/result/*
                    do
                        $PYTHON $SRC/report.py $res 
                        let i++
                    	cp -R $SRC/js $res
                    	cp -R $SRC/css $res
                    done
                fi
                echo "Time - report:"
                date
	    fi
	fi
    fi
fi
