#!/usr/bin/env perl
#!/bin/env perl

## Name: getcommontsc.pl
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# get common UNS transcripts from two gff files
# --input, input gff file with UNS genes and their corresponding exon
# --reference, input gff file for comparision
# --exon, print exons
# --output, output common gff file
# --diff, output diff gff file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $reference;
my $exon;
my $output = "output.gff";
my $diff = "diff.gff";
my $linein;
my $linecomp;
my @argsin;
my @argscomp;
$argsin[3] = -1;
$argscomp[3] = -1;

usage() if (@ARGV < 4 or
            ! GetOptions("help|?" => \$help,
			 "exon" => \$exon,
                         "input=s" => \$input,
			 "reference=s" => \$reference,
			 "diff=s" => \$diff,
			 "output=s" => \$output)
            or defined $help);

open(IN, $input) or die("Input file not found\n");
open(REF, $reference) or die("Reference file not found\n");
open(OUT, ">$output");
open(DIFF, ">$diff");

$linein = <IN>;

#while ($linein = <IN>) {
while ($linein) {
    chomp($linein);
    #< get id of UNS transcript
    @argsin = split("\t", $linein);
    #>

    #< search input id reference
    do {
	$linecomp = <REF>;
	if ($linecomp) {
	    chomp($linecomp);
	    @argscomp = split("\t", $linecomp);
	}
    } until (!$linecomp || $argscomp[3] >= $argsin[3]);
    #>
    
    #< if found then print
    if ($argscomp[3] == $argsin[3]) {
	print OUT "$linein\n";
	#< print exon
	if ($exon) {
	    $linecomp = <REF>;
	    $linein = <IN>;
	    print OUT "$linein";
	}
	#>
	$linein = <IN>;
    }
    #>
    #< otherwise, search reference id in input
    else {
	do {
	    print DIFF "$linein\n";
	    $linein = <IN>;
	    if ($linein) {
		chomp($linein);
		#< get id of UNS transcript
		@argsin = split("\t", $linein);
		#>
	    }
	} until (!$linein || $argsin[3] >= $argscomp[3]);
	
	#< if found then print
	if ($argscomp[3] == $argsin[3]) {
	    print OUT "$linein\n";
	    #< print exon
	    if ($exon) {
		$linecomp = <REF>;
		$linein = <IN>;
		print OUT "$linein";
	    }
	    $linein = <IN>;
	    #>
	}
	#>
    }
    #>
}

close(IN);
close(REF);
close(OUT);
close(DIFF);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl getcommontsc.pl --input GFF --reference REF [--exon] [--output OUT] [--help|?] \n";
    exit 1;
}
