#!/usr/bin/env perl
#!/bin/env perl

## Name: gettsc.pl
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# retrieve transcripts with features of request and convert all features to 'gene'
# --input, input gff file containing at least genes and their exons
# --feature, file containing transcript features of request
## e.g.: 
## gene
## pseudogene
# --strand, +/-
# --output, output gff file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $feature;
my $strand=".";
my $output="output.gff";

my $line;
my @args;
my @argsID;
my @argsName;
my @features = ("gene","pseudogene");
my $exist;
my $i;

usage() if ($#ARGV < 3 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "feature=s" => \$feature,
			 "strand=s" => \$strand,
			 "output=s" => \$output)
	    or defined $help
	    or ! defined $strand);

if ($feature) {
    open(FEAT, $feature) or die("Feature file not found\n");
    @features = ();
    while ($line = <FEAT>) {
	chomp($line);
	push(@features, $line);
    }
    close(FEAT);
}

open(OUT, ">$output");
open(GFF, $input) or die("GFF file not found\n");
while ($line = <GFF>) {
    chomp($line);
    @args = split("\t", $line);
    
    if (substr($line, 0, 1) ne "#" && $#args > 5 && ($args[6] eq $strand)) {
	#< print all exons
	if (lc($args[2]) eq "exon" || lc($args[2]) eq "pseudogenic_exon") {
	    $args[2] = "exon";
	    for ($i=0; $i<$#args; $i++) {
		print OUT "$args[$i]\t";
	    }
	    if (index($args[$i], "arent")!=-1) {
		@argsID = split("arent", $args[$i]);
		@argsID = split(";", $argsID[1]);
		$argsID[0] = &trim($argsID[0]);
		print OUT "Parent=$argsID[0]\n";
	    }
	    elsif (index($args[$i], "ID")!=-1) {
		@argsID = split("ID", $args[$i]);
		@argsID = split(";", $argsID[1]);
		$argsID[0] = &trim($argsID[0]);
		print OUT "Parent=$argsID[0]\n";
	    }
	    else {
		print OUT "$args[$i]\n";
	    }
	}
	#>
	else {
	    $exist = 0;
	    #< check if the feature of line exists in the given list
	    for ($i=0; $i<=$#features; $i++) {
		if (lc($args[2]) eq lc($features[$i])) {
		    $exist = 1;
		    last;
		}
	    }
	    #>
	    #< if exist, replace by "gene"
	    if ($exist==1) {
		$args[2] = "gene";
		for ($i=0; $i<$#args; $i++) {
		    print OUT "$args[$i]\t";
		}
		@argsID = split("ID", $args[$i]);
		@argsID = split(";", $argsID[1]);
		$argsID[0] = &trim($argsID[0]);
		print OUT "ID=$argsID[0];";
		if (index($args[$i], "Name")!=-1) {
		    @argsName = split("Name", $args[$i]);
		    @argsName = split(";", $argsName[1]);
		    $argsName[0] = &trim($argsName[0]);
		    print OUT "Name=$argsName[0]";
		}
		print OUT "\n";
	    }
	    #>
	}
    }
}
close(GFF);
close(OUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl gettsc.pl --input GFF --feature FEAT --strand +/- [--output OUT] [--help|?]\n";
    exit 1;
}

sub trim {
    my $str = shift;
    
    ##-- remove white spaces at the beginning and at the end
    $str =~ s/^\s+|\s+$//g;
    ##-- remove quotes at the beginning and at the end
    $str =~ s/^\"+|\"+$//g;
    ##-- remove ='s at the beginning and at the end
    $str =~ s/^\=+|\=+$//g;
    return $str;
}
