#!/usr/bin/env perl
#!/bin/env perl

## Name: separatestrand.pl
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## separate sam into negative, positive and unstranded reads
# --input, input sam file
# --neg, output negative sam
# --pos, output positive sam
# --uns, output unstranded sam

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $neg = "NEG.sam";
my $pos = "POS.sam";
my $uns = "UNS.sam";

my $line;
my $xs;
my @args;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "neg=s" => \$neg,
			 "pos=s" => \$pos,
			 "uns=s" => \$uns)
	    or defined $help);

if (defined $input) {
    open(STDIN, $input);
}
open(NEG, ">$neg");
open(POS, ">$pos");
open(UNS, ">$uns");

while ($line = <STDIN>) {
    chomp($line);
    @args = split("\t", $line);
    if ($args[0] eq "\@HD" || $args[0] eq "\@SQ" || $args[0] eq "\@PG") {
        print POS "$line\n";
        print NEG "$line\n";
	print UNS "$line\n";
    }
    else {
        last;
    }
}
if (!$line) {
    exit 0;
}
do {
    chomp($line);
    $xs = "XS:A:+";
    if (index($line, $xs) != -1) {
	print POS "$line\n";
    }
    else {
	$xs = "XS:A:-";
	if (index($line, $xs) != -1) {
	    print NEG "$line\n";
	}
	else {
	    print UNS "$line\n";
	}
    }
} while ($line=<STDIN>);

close(STDIN);
close(POS);
close(NEG);
close(UNS);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl separatestrand.pl --input GFF [--neg NEG] [--pos POS] [--uns UNS] [--help|?] \n";
    exit 1;
}
