#!/usr/bin/env perl
#!/bin/env perl

## Name: unscomparechr.pl
## Version: 1.1.6
## Date: 01-01-2015
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## cut unstranded transcripts non-overlapped with stranded reference transcripts
# --input, input gff file
# --reference, reference gff file
# --output, output gff file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $reference;
my $output = "output.gff";
my $linenew;
my $lineref;
my @argsref;
my @argsnew;
my @noteref;
my @notenew;
my @notenew_noexon;
my $index = 0;
my $endnew;
my $i;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "reference=s" => \$reference,
			 "output=s" => \$output)
	    or defined $help);

open(REF, $reference) or die("Reference file not found\n");
open(IN, $input) or die("Input file not found\n");
open(OUT, ">$output");

$argsnew[4] = -1;

 WHOLE: {
     while ($lineref = <REF>) {
	 chomp($lineref);
	 @argsref = split("\t", $lineref);
	 @noteref = split(";", $argsref[8]);#exons=
	 
	 #< intergenic: all new transcripts coming before the current reference 
	 while ($argsnew[4] < $argsref[3]) {
	     if ($argsnew[4] > -1) {
		 @notenew = split("exons=", $argsnew[8]);
		 $notenew[1] = "$argsnew[3] $argsnew[4]";
		 for ($i=0; $i<8; $i++) {
		     print OUT "$argsnew[$i]\t";
		 }
		 @notenew_noexon = split(";", $notenew[0]);
		 $index = $index + 1;
		 print OUT "$notenew_noexon[0].";
		 print OUT "$index;";
		 for ($i=1; $i<$#notenew_noexon; $i++) {
		     print OUT "$notenew_noexon[$i];";
		 }
		 print OUT " exons=$notenew[1]\n";
	     }
	     $linenew = <IN>;
	     $index = 0;
	     if (!$linenew) {
		 last WHOLE;
	     }
	     chomp($linenew);
	     @argsnew = split("\t", $linenew);
	 }
	 #>
	 
	 #< overlapped: all new transcripts overlapping with the current reference
	 if ($argsnew[3] < $argsref[3]) {
	     $endnew = $argsnew[4];
	     $argsnew[4] = $argsref[3];
	     @notenew = split("exons=", $argsnew[8]);
	     $notenew[1] = "$argsnew[3] $argsnew[4]";
	     for ($i=0; $i<8; $i++) {
		 print OUT "$argsnew[$i]\t";
	     }
	     @notenew_noexon = split(";", $notenew[0]);
	     $index = $index + 1;
	     print OUT "$notenew_noexon[0].";
	     print OUT "$index;";
	     for ($i=1; $i<$#notenew_noexon; $i++) {
		 print OUT "$notenew_noexon[$i];";
	     }
	     print OUT " exons=$notenew[1]\n";
	     if ($endnew <= $argsref[4]) {
		 $linenew = <IN>;
		 $index = 0;
		 if (!$linenew) {
		     last WHOLE;
		 }
		 chomp($linenew);
		 @argsnew = split("\t", $linenew);
	     }
	     else {
		 $argsnew[3] = $argsref[4];
		 $argsnew[4] = $endnew;
	     }
	 }
	 #>

	 #< included: all new transcripts included in the current reference
	 while ($argsnew[4] <= $argsref[4]) {
	     $linenew = <IN>;
	     $index = 0;
	     if (!$linenew) {
		 last WHOLE;
	     }
	     chomp($linenew);
	     @argsnew = split("\t", $linenew);
	 }
	 #>
	 print OUT "$lineref\n";
	 
	 #< overlapped: all new transcripts overlapping with the current reference
	 if ($argsnew[3] < $argsref[4]) {
	     $argsnew[3] = $argsref[4];
	 }
	 #>
     }
}

#< last references
do {
    if ($lineref) {
	print OUT "$lineref\n";
    }
    $lineref = <REF>;
    if ($lineref) {
	chomp($lineref);
    }
} until (!$lineref);
#>

#< transcripts coming after the last reference
do {
    if ($linenew) {
	@notenew = split("exons=", $argsnew[8]);
	$notenew[1] = "$argsnew[3] $argsnew[4]";
	for ($i=0; $i<8; $i++) {
	    print OUT "$argsnew[$i]\t";
	}
	@notenew_noexon = split(";", $notenew[0]);
	$index = $index + 1;
	print OUT "$notenew_noexon[0].";
	print OUT "$index;";
	for ($i=1; $i<$#notenew_noexon; $i++) {
	    print OUT "$notenew_noexon[$i];";
	}
	print OUT " exons=$notenew[1]\n";
    }
    $linenew = <IN>;
    $index = 0;
    if ($linenew) {
	chomp($linenew);
	@argsnew = split("\t", $linenew);
    }
} until (!$linenew && eof(IN));
#>

close(REF);
close(IN);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl unscomparechr.pl --input GFF --reference REF [--output OUT] [--help|?] \n";
    exit 1;
}
